package com.gwtorbit.client.widgets;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.DoubleClickEvent;
import com.google.gwt.event.dom.client.DoubleClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.dom.client.HasDoubleClickHandlers;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

public class BSelectPanel extends VerticalPanel implements ClickHandler, DoubleClickHandler {

	public final static String ITEM_SELECTED_STYLE = "BSelectPanelItemSelected";

	public static interface ItemClickedHandler {
		void onClick(Widget w);
	}

	public static interface ItemDoubleClickedHandler {
		void onDoubleClick(Widget w);
	}

	private HashSet<Widget> selectedSet = new HashSet<Widget>();

	private HashMap<Widget, HandlerRegistration> clickHRs = new HashMap<Widget, HandlerRegistration>();

	private HashMap<Widget, HandlerRegistration> doubleClickHRs = new HashMap<Widget, HandlerRegistration>();

	private ArrayList<ItemDoubleClickedHandler> itemDoubleClickedHandlers = new ArrayList<ItemDoubleClickedHandler>();

	private ArrayList<ItemClickedHandler> itemClickedHandlers = new ArrayList<ItemClickedHandler>();

	public BSelectPanel() {
	}

	public void add(Widget w) {
		super.add(w);
		if (w instanceof HasClickHandlers) {
			clickHRs.put(w, ((HasClickHandlers) w).addClickHandler(this));
		}
		if (w instanceof HasDoubleClickHandlers) {
			doubleClickHRs.put(w, ((HasDoubleClickHandlers) w).addDoubleClickHandler(this));
		}
	}

	public void clear() {
		while (getWidgetCount() > 0) {
			Widget widget = getWidget(0);
			remove(widget);
		}
	}

	public Iterator<Widget> iterator() {
		return super.iterator();
	}

	public boolean remove(Widget w) {
		removeFromSelected(w);

		if (clickHRs.containsKey(w)) {
			clickHRs.remove(w).removeHandler();
		}
		if (doubleClickHRs.containsKey(w)) {
			doubleClickHRs.remove(w).removeHandler();
		}
		return super.remove(w);
	}

	public void onClick(ClickEvent event) {
		Widget widget = (Widget) event.getSource();

		if (event.getNativeEvent().getCtrlKey()) {
			if (selectedSet.contains(widget)) {
				removeFromSelected(widget);
			} else {
				addToSelected(widget);
			}
		} else {
			clearSelected();
			addToSelected((Widget) event.getSource());
		}

		for (ItemClickedHandler itemClickedHandler : itemClickedHandlers) {
			itemClickedHandler.onClick(widget);
		}
	}

	public void onDoubleClick(DoubleClickEvent event) {
		for (ItemDoubleClickedHandler itemDoubleClickedHandler : itemDoubleClickedHandlers) {
			itemDoubleClickedHandler.onDoubleClick((Widget) event.getSource());
		}
	}

	public void clearSelected() {
		ArrayList<Widget> widgets = new ArrayList<Widget>(selectedSet);
		for (Widget widget : widgets) {
			removeFromSelected(widget);
		}
	}

	public void removeFromSelected(Widget widget) {
		if (selectedSet.contains(widget)) {
			selectedSet.remove(widget);
			widget.removeStyleName(ITEM_SELECTED_STYLE);
		}
	}

	public void addToSelected(Widget widget) {
		selectedSet.add(widget);
		widget.addStyleName(ITEM_SELECTED_STYLE);
	}
	
	public ArrayList<Widget> getSelected() {
		return new ArrayList<Widget>(selectedSet);
	}

	public ArrayList<Widget> removeSelected() {
		ArrayList<Widget> widgets = new ArrayList<Widget>(selectedSet);
		for (Widget widget : widgets) {
			remove(widget);
		}
		return widgets;
	}

	public void addItemDoubleClickedHandler(ItemDoubleClickedHandler handler) {
		itemDoubleClickedHandlers.add(handler);
	}

	public void removeItemDoubleClickedHandler(ItemDoubleClickedHandler handler) {
		itemDoubleClickedHandlers.remove(handler);
	}

	public void clearItemDoubleClickedHandler() {
		itemDoubleClickedHandlers.clear();
	}

	public void addItemClickedHandler(ItemClickedHandler handler) {
		itemClickedHandlers.add(handler);
	}

	public void removeItemClickedHandler(ItemClickedHandler handler) {
		itemClickedHandlers.remove(handler);
	}

	public void clearItemClickedHandler() {
		itemClickedHandlers.clear();
	}

}
