/*
 * Copyright (C) 2010 David Arkell.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.gwtorbit.client.widgets;

import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.gwtorbit.client.model.UiData;
import com.gwtorbit.client.model.UiDataDisplay;
import com.gwtorbit.client.model.UiErrors;

public class FormUiCheckBox extends FormAbstractRowWithUiErrorDisplay implements UiDataDisplay,
		HasClickHandlers {

	private UiCheckBox field;

	public FormUiCheckBox() {
		field = new UiCheckBox();
		setFieldWidget(field);
	}

	public void setEnabled(boolean enabled) {
		field.setEnabled(enabled);
	}

	public boolean isEnabled() {
		return field.isEnabled();
	}

	public HandlerRegistration addKeyUpHandler(KeyUpHandler handler) {
		return field.addKeyUpHandler(handler);
	}

	public void setFocus(boolean focused) {
		field.setFocus(focused);
	}

	public HandlerRegistration addClickHandler(ClickHandler handler) {
		return field.addClickHandler(handler);
	}

	public boolean getValue() {
		return field.getValue();
	}

	public void setValue(boolean value) {
		field.setValue(value);
	}

	public void setDataName(String dataName) {
		super.setDataName(dataName);
		field.setDataName(dataName);
	}

	public void displayErrors(UiErrors errors, String errorPrefix) {
		super.displayErrors(errors, errorPrefix);
		field.displayErrors(errors, errorPrefix);
	}

	public void displayData(UiData uiData) {
		field.displayData(uiData);
	}

	public void getData(UiData uiData, UiErrors errors, String errorPrefix) {
		field.getData(uiData, errors, errorPrefix);
	}

	public void clearErrors() {
		super.clearErrors();
		field.clearErrors();
	}

	public void clearData() {
		field.clearData();
	}

	public boolean getIgnore() {
		return field.getIgnore();
	}

	public void setIgnore(boolean ignore) {
		field.setIgnore(ignore);
	}

}
