/*
 * Copyright (C) 2010 David Arkell.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.gwtorbit.client.widgets;

import com.google.gwt.user.client.ui.RichTextArea;
import com.google.gwt.user.client.ui.TextArea;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.gwtorbit.client.model.UiData;
import com.gwtorbit.client.model.UiDataDisplay;
import com.gwtorbit.client.model.UiErrors;
import com.gwtorbit.client.model.UiLocalizedError;

public class UiRichTextArea extends VerticalPanel implements UiDataDisplay {

	private RichTextArea richTextArea;
	
	private TextArea textArea;
	
	private boolean wysiwygDisplayed = true;
	
	public UiRichTextArea() {
		richTextArea = new RichTextArea();
		textArea = new TextArea();
		textArea.setVisible(false);
		
		add(richTextArea);
		add(textArea);
		
		setWidth("400px");
		setHeight("200px");
	}
	
	public void setWidth(String width) {
		super.setWidth(width);
		richTextArea.setWidth(width);
		textArea.setWidth(width);
	}

	public void setHeight(String height) {
		super.setHeight(height);
		richTextArea.setHeight(height);
		textArea.setHeight(height);
	}
	
	public void setWysiwygDisplayed(boolean wysiwygDisplayed) {
		if (this.wysiwygDisplayed != wysiwygDisplayed) {
			this.wysiwygDisplayed = wysiwygDisplayed;
			
			if (this.wysiwygDisplayed) {
				richTextArea.setHTML(textArea.getText());
			} else {
				textArea.setText(richTextArea.getHTML());
			}
		}
		updateDisplay();
	}
	
	private void updateDisplay() {
		textArea.setVisible(!wysiwygDisplayed);
		richTextArea.setVisible(wysiwygDisplayed);
	}
	
	public void setHTML(String html) {
		if (wysiwygDisplayed) {
			richTextArea.setHTML(html);
		} else {
			textArea.setText(html);
		}
	}
	
	public String getHTML() {
		if (wysiwygDisplayed) {
			return richTextArea.getHTML();
		} else {
			return textArea.getText();
		}
	}
	
	private boolean enabled = true;
	
	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
		
		richTextArea.setEnabled(enabled);
		textArea.setEnabled(enabled);
	}

	public boolean isEnabled() {
		return enabled;
	}
	
	public RichTextArea getRichTextArea() {
		return richTextArea;
	}
	
	public TextArea getTextArea() {
		return textArea;
	}

	public void displayData(UiData uiData) {
		if (ignore) {
			return;
		}

		setHTML((String) uiData.getData(dataName));
	}

	public void getData(UiData uiData, UiErrors errors, String errorPrefix) {
		if (ignore) {
			return;
		}

		String text = getHTML();

		if (mandatory && "".equals(text)) {
			errors.addError(errorPrefix + dataName, UiLocalizedError.MANDATORY);
		}

		uiData.setData(dataName, text);
	}

	public void clearData() {
		if (ignore) {
			return;
		}

		setHTML("");
	}

	private boolean mandatory = false;

	private String dataName;

	private boolean ignore = false;

	public String getDataName() {
		return dataName;
	}

	public void setDataName(String dataName) {
		this.dataName = dataName;
	}

	public void setMandatory(boolean mandatory) {
		this.mandatory = mandatory;
	}

	public boolean isMandatory() {
		return mandatory;
	}

	public boolean getIgnore() {
		return ignore;
	}

	public void setIgnore(boolean ignore) {
		this.ignore = ignore;
	}

}
