/*
 * Copyright (C) 2010 David Arkell.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.gwtorbit.client.widgets;

import java.util.Date;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.datepicker.client.DateBox;
import com.google.gwt.user.datepicker.client.DateBox.Format;
import com.gwtorbit.client.model.UiData;
import com.gwtorbit.client.model.UiDataDisplay;
import com.gwtorbit.client.model.UiErrorDisplay;
import com.gwtorbit.client.model.UiErrors;
import com.gwtorbit.client.model.UiLocalizedError;
import com.gwtorbit.client.mvp.ClientResourceBundle;

public class UiTextBoxDateOrYear extends HorizontalPanel implements UiDataDisplay, UiErrorDisplay {

	private DateBox dateBox;
	
	private TextBox yearTextBox;
	
	private LLink showDate;
	
	private LLink showYear;
	
	private boolean dateShown = true;
	
	private DateTimeFormat eraFormat = DateTimeFormat.getFormat("G");
	
	private DateTimeFormat yearFormat = DateTimeFormat.getFormat("yyyy");
	
	private DateTimeFormat yearAndEraFormat = DateTimeFormat.getFormat("yyyy-MM-dd G");
	
	public UiTextBoxDateOrYear() {
		dateBox = new DateBox();
		
		dateBox.setFormat(new DateBox.DefaultFormat(ClientResourceBundle.dateFormat));
		dateBox.setStyleName("UiTextBoxShort");
		add(dateBox);
		
		yearTextBox = new TextBox();
		add(yearTextBox);
		yearTextBox.setVisible(false);
		
		showYear = new LLink("UiTextBoxDateOrYear.showYear");
		add(showYear);
		
		showDate = new LLink("UiTextBoxDateOrYear.showDate");
		showDate.setVisible(false);
		add(showDate);
		
		showYear.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				setDateShown(false);
			}
		});
		
		showDate.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				setDateShown(true);
			}
		});
	}
	
	public void setDateShown(boolean dateShown) {
		this.dateShown = dateShown;
		updateDisplay();
	}
	
	public boolean setDateShown() {
		return dateShown;
	}
	
	private void updateDisplay() {
		dateBox.setVisible(dateShown);
		yearTextBox.setVisible(!dateShown);
		
		showYear.setVisible(dateShown);
		showDate.setVisible(!dateShown);
		
		if (dateShown) {
			yearTextBox.setText("");
		} else {
			dateBox.getTextBox().setText("");
		}
	}

	public void displayData(UiData uiData) {
		if (ignore) {
			return;
		}

		setValue((Date) uiData.getData(dataName));
	}
	
	public void setValue(Date value) {
		if (value == null) {
			dateBox.getTextBox().setText("");
			yearTextBox.setText("");		
			return;
		}
		
		if ("BC".equals(eraFormat.format(value))) {
			setDateShown(false);
			
			yearTextBox.setText("-" + yearFormat.format(value));
		} else {
			setDateShown(true);
			dateBox.setValue(value);
		}
	}
	
	public Date getValue() {
		if (dateShown) {
			return dateBox.getValue();
		} else {
			if (yearTextBox.getValue().equals("")) {
				return null;
			}
			
			try {
				int year = (int) ClientResourceBundle.integerFormat.parse(yearTextBox.getValue());

				Date date = new Date();
				if (year < 0) {
					date.setYear(year - 1899);
				} else {
					date.setYear(year - 1900);
				}
				date.setMonth(0);
				date.setDate(1);
				date.setHours(0);
				date.setMinutes(0);
				date.setSeconds(0);
				return date;
				
//				String yearAsString = null;
//				if (year < 0) {
//					yearAsString = "" + (-year) + " BC";
//				} else {
//					yearAsString = "" + year + "-01-01 AD";
//				}
//				Window.alert(yearAsString);
//				return yearAndEraFormat.parse(yearAsString);
			} catch (NumberFormatException e) {
//			} catch (IllegalArgumentException e2) {
//				Window.alert("aaa");
			}
			
			
			return null;
		}
	}
	
	public void getData(UiData uiData, UiErrors errors, String errorPrefix) {
		if (ignore) {
			return;
		}

		Date date = getValue();
		String text = dateBox.getTextBox().getText();
		if (!dateShown) {
			text = yearTextBox.getText();
		}

		if (mandatory && "".equals(text)) {
			errors.addError(errorPrefix + dataName, UiLocalizedError.MANDATORY);
		}

		if (!"".equals(text) && date == null) {
			errors.addError(errorPrefix + dataName, UiLocalizedError.INVALID);
		}

		uiData.setData(dataName, date);
	}

	public void clearData() {
		if (ignore) {
			return;
		}

		setValue(null);
	}

	private boolean mandatory = false;

	private String dataName;

	private boolean ignore = false;

	public String getDataName() {
		return dataName;
	}

	public void setDataName(String dataName) {
		this.dataName = dataName;
	}

	public void setMandatory(boolean mandatory) {
		this.mandatory = mandatory;
	}

	public boolean isMandatory() {
		return mandatory;
	}

	public void clearErrors() {
		this.removeStyleName("UiTextBoxError");
	}

	public void displayErrors(UiErrors errors, String errorPrefix) {
		if (errors.hasErrors(errorPrefix + dataName)) {
			this.addStyleName("UiTextBoxError");
		}
	}

	public boolean getIgnore() {
		return ignore;
	}

	public void setIgnore(boolean ignore) {
		this.ignore = ignore;
	}

	public boolean isEnabled() {
		return dateBox.getTextBox().isEnabled();
	}

	public void setEnabled(boolean enabled) {
		dateBox.setEnabled(enabled);
		yearTextBox.setEnabled(enabled);
	}
	
	public void setFocus(boolean focus) {
		if (dateShown) {
			dateBox.setFocus(focus);
		} else {
			yearTextBox.setFocus(focus);
		}
	}
}
