/*
 * Copyright (C) 2010 David Arkell.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.gwtorbit.client.widgets;

import java.util.ArrayList;

import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.gwtorbit.client.model.DisplayMethods;
import com.gwtorbit.client.model.UiData;
import com.gwtorbit.client.model.UiDataDisplay;
import com.gwtorbit.client.model.UiErrorDisplay;
import com.gwtorbit.client.model.UiErrors;
import com.gwtorbit.client.model.UiLocalizedError;

public abstract class UiVerticalList extends VerticalPanel implements UiDataDisplay, UiErrorDisplay {

	protected abstract Widget createDisplayInstance(UiData uiData);

	protected abstract UiData createDataInstance(Widget w);

	@SuppressWarnings("unchecked")
	public void displayData(UiData uiData) {
		ArrayList<UiData> dataList = (ArrayList<UiData>) uiData.getData(dataName);

		clearData();
		for (UiData tmpData : dataList) {
			addItem(tmpData);
		}
	}

	public void getData(UiData uiData, UiErrors errors, String errorPrefix) {
		ArrayList<UiData> dataList = new ArrayList<UiData>();

		for (int i = 0; i < getWidgetCount(); i++) {
			Widget tmpDisplay = getWidget(i);
			UiData tmpData = createDataInstance(tmpDisplay);

			DisplayMethods.getData(tmpDisplay, tmpData, errors, errorPrefix + dataName + "[" + i
					+ "].");
			dataList.add(tmpData);
		}

		if (mandatory && getWidgetCount() == 0) {
			errors.addError(errorPrefix + dataName, UiLocalizedError.MANDATORY);
		}

		uiData.setData(dataName, dataList);
	}

	public void clearData() {
		this.clear();
	}

	private String dataName;

	private boolean ignore = false;

	private boolean mandatory = false;

	public String getDataName() {
		return dataName;
	}

	public void setDataName(String dataName) {
		this.dataName = dataName;
	}

	public void clearErrors() {
		for (int i = 0; i < this.getWidgetCount(); i++) {
			Widget tmpDisplay = this.getWidget(i);
			DisplayMethods.clearErrors(tmpDisplay);
		}
	}

	public void displayErrors(UiErrors errors, String errorPrefix) {
		for (int i = 0; i < getWidgetCount(); i++) {
			Widget tmpDisplay = getWidget(i);
			DisplayMethods.displayErrors(tmpDisplay, errors, errorPrefix + dataName + "[" + i
					+ "].");
		}
	}

	public Widget addItem(UiData uiData) {
		Widget widget = createDisplayInstance(uiData);
		addItem(widget, uiData);
		return widget;
	}

	public void addItem(Widget widget, UiData uiData) {
		this.add(widget);
		DisplayMethods.displayData(widget, uiData);
	}

	public boolean getIgnore() {
		return ignore;
	}

	public void setIgnore(boolean ignore) {
		this.ignore = ignore;
	}

	public void setMandatory(boolean mandatory) {
		this.mandatory = mandatory;
	}

	public boolean isMandatory() {
		return mandatory;
	}

}
